import hyperion
import socket
import errno
import struct

# Get the parameters
ListenPort = int(hyperion.args.get('port', 21328))
ListenIP = hyperion.args.get('host', "")
octets = ListenIP.split('.');

scriptname = "udpDRGBListener.py"

UDPSock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM, socket.IPPROTO_UDP)
UDPSock.setblocking(False)

listen_addr = (ListenIP,ListenPort)
UDPSock.bind(listen_addr)

if ListenIP == "":
        print (scriptname,"- INFO: Listening on *.*.*.*:",str(ListenPort))
else:
        print (scriptname,"- INFO: Listening on "+ListenIP+":",str(ListenPort))

if len(octets) == 4 and int(octets[0]) >= 224 and int(octets[0]) < 240:
        print (scriptname,"- INFO: ListenIP is a multicast address\n")
        # Multicast handling
        try:
                mreq = struct.pack("4sl", socket.inet_aton(ListenIP), socket.INADDR_ANY)
                UDPSock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
        except socket.error:
                print (scriptname,"- ERROR enabling multicast\n")

hyperion.setColor(hyperion.ledCount *  bytearray((int(0), int(0), int(0))) )

# Start the write data loop
while not hyperion.abort():
                try:
                        data,addr = UDPSock.recvfrom(4500)
                        #print (data.strip(),len(data),addr)
                        if (len(data)%3 == 2 and data[0] == 2):
                                #print ("DRGB protocol:", data[0], " timeout:", data[0], "s" )
                                #Strip 2 Bytes to get LED color information
                                data = data[2:]
                                #print (data.strip(),len(data),addr)
                                if (len(data)%3 == 0):
                                        #print ("numleds ",len(data)/3)
                                        ledData = bytearray()
                                        for i in range(hyperion.ledCount):
                                                if (i<(len(data)/3)):
                                                        ledData.append(data[i*3+0])
                                                        ledData.append(data[i*3+1])
                                                        ledData.append(data[i*3+2])
                                                else:
                                                        ledData += bytearray((int(0), int(0), int(0)))

                                        hyperion.setColor(ledData)
                                else:
                                        print (scriptname,"- ERROR: LED color information is not devidable by 3 and connot be used")
                        else:
                                print (scriptname,"- ERROR: Only DRGB protocol is supported")
                except IOError as e:
                        if e.errno == errno.EWOULDBLOCK:
                                pass
                        else:
                                print (scriptname,"- ERROR: errno:", e.errno)

print (scriptname,"- INFO: Closing socket")
UDPSock.close()

